import httpx

DEFAULT_MAX_NR_CONNECTIONS = 10
NETWORK_CONNECTIONS_FACTOR = 5


def get_network_limits(max_nr_connections: int = DEFAULT_MAX_NR_CONNECTIONS) -> httpx.Limits:
    max_keepalive_connections = int(max_nr_connections / NETWORK_CONNECTIONS_FACTOR)
    if max_keepalive_connections <= 0:
        max_keepalive_connections = 1
    return httpx.Limits(max_keepalive_connections=max_keepalive_connections, max_connections=max_nr_connections)


def get_network_timeouts(timeout: int = 60) -> httpx.Timeout:
    return httpx.Timeout(timeout, pool=None, write=3600)



from typing import Dict, Optional, Union

from pydantic import BaseModel, ConfigDict
from pydantic.alias_generators import to_camel


class BaseSerializer(BaseModel):
    model_config = ConfigDict(populate_by_name=True, alias_generator=to_camel, from_attributes=True)

    @classmethod
    def from_json(cls, js: Dict):
        return cls.model_validate(js)

    def to_dict(self, by_alias=True) -> Dict:
        return self.model_dump(exclude_none=True, by_alias=by_alias)


CursorIdType = Union[int, str]


class PageMetadata(BaseSerializer):
    next_cursor_id: Optional[CursorIdType] = None


class PaginatedResponse(BaseSerializer):
    data: Union[dict, list]
    metadata: PageMetadata = PageMetadata()



from kognic.base_clients.cloud_storage.network_limits import get_network_limits


def test_get_network_limits_with_10():
    limits = get_network_limits(10)
    assert limits.max_keepalive_connections == 2
    assert limits.max_connections == 10


def test_get_network_limits_with_5():
    limits = get_network_limits(5)
    assert limits.max_keepalive_connections == 1
    assert limits.max_connections == 5


def test_get_network_limits_with_100():
    limits = get_network_limits(100)
    assert limits.max_keepalive_connections == 20
    assert limits.max_connections == 100


def test_get_network_limits_with_4():
    limits = get_network_limits(4)
    assert limits.max_keepalive_connections == 1
    assert limits.max_connections == 4


def test_get_network_limits_with_1():
    limits = get_network_limits(1)
    assert limits.max_keepalive_connections == 1
    assert limits.max_connections == 1
